@@ -5,9 +5,9 @@ Lazy::Utils - Utility functions
 
 =head1 VERSION
 
-version 1.14
+version 1.15
 
-=head1 ABSTRACT
+=head1 SYNOPSIS
 
 Utility functions
 
@@ -19,13 +19,13 @@ Utility functions
 	file_get_contents($path, $prefs);
 	file_put_contents($path, $contents, $prefs);
 	shellmeta($s, $nonquoted);
-	_system($cmd, @argv);
-	bashReadLine($prompt);
-	commandArgs($prefs, @argv);
-	cmdArgs(@argv);
-	whereisBin($name, $path);
-	fileCache($tag, $expiry, $subref);
-	getPodText($fileName, $section, $exclude_section);
+	alt_system($cmd, @argv);
+	bash_readline($prompt);
+	cmdargs($prefs, @argv);
+	whereis($name, $path);
+	file_cache($tag, $expiry, $subref);
+	get_pod_text($file_name, $section, $exclude_section);
+	term_readline($prompt, $default, $history);
 
 =head1 DESCRIPTION
 
@@ -35,17 +35,22 @@ Collection of utility functions all of exported by default
 use strict;
 use warnings;
 use v5.10.1;
+use feature qw(switch);
+no if ($] >= 5.018), 'warnings' => 'experimental';
 use FindBin;
 use JSON;
 use Pod::Simple::Text;
+use Term::ReadKey;
 
 
 BEGIN
 {
 	require Exporter;
-	our $VERSION     = '1.14';
+	our $VERSION     = '1.15';
 	our @ISA         = qw(Exporter);
-	our @EXPORT      = qw(trim ltrim rtrim file_get_contents file_put_contents shellmeta _system bashReadLine commandArgs cmdArgs whereisBin fileCache getPodText);
+	our @EXPORT      = qw(trim ltrim rtrim file_get_contents file_put_contents shellmeta alt_system _system
+		bash_readline bashReadLine cmdargs commandArgs cmdArgs whereis whereisBin file_cache fileCache
+		get_pod_text getPodText term_readline);
 	our @EXPORT_OK   = qw();
 }
 
@@ -106,7 +111,7 @@ gets all contents of file in string type
 
 $path: I<path of file>
 
-$prefs: I<preferences in hash type, by default undef>
+$prefs: I<preferences in HashRef, by default undef>
 
 =over
 
@@ -142,7 +147,7 @@ $path: I<path of file>
 
 $contents: I<file contents in string type>
 
-$prefs: I<preferences in hash type, by default undef>
+$prefs: I<preferences in HashRef, by default undef>
 
 =over
 
@@ -191,9 +196,11 @@ sub shellmeta
 	return $s;
 }
 
-=head3 _system($cmd, @argv)
+=head3 alt_system($cmd, @argv)
 
-executes a system command like Perl system call
+B<_system($cmd, @argv)> I<WILL BE DEPRECATED>
+
+alternative implementation of perls core system subroutine that executes a system command
 
 $cmd: I<command>
 
@@ -201,12 +208,12 @@ $cmd: I<command>
 
 return value: I<exit code of command. 511 if fatal error occurs>
 
-returned $?: I<return code of wait call like on Perl system call>
+returned $?: I<return code of wait call like on perls system call>
 
-returned $!: I<system error message like on Perl system call>
+returned $!: I<system error message like on perls system call>
 
 =cut
-sub _system
+sub alt_system
 {
 	my $pid;
 	if (not defined($pid = fork))
@@ -225,32 +232,47 @@ sub _system
 	}
 	return $? >> 8;
 }
+sub _system
+{
+	return alt_system(@_);
+}
 
-=head3 bashReadLine($prompt)
+=head3 bash_readline($prompt)
 
-reads a line using bash
+B<bashReadLine($prompt)> I<WILL BE DEPRECATED>
+
+reads a line from STDIN using Bash
 
 $prompt: I<prompt, by default ''>
 
 return value: I<line>
 
 =cut
-sub bashReadLine
+sub bash_readline
 {
 	my ($prompt) = @_;
-	unless ( -t *STDIN )
+	$prompt = "" unless defined($prompt);
+	unless (-t *STDIN)
 	{
-		my $line = <STDIN>;
+		my $line = <*STDIN>;
 		chomp $line if defined $line;
 		return $line;
 	}
-	$prompt = "" unless defined($prompt);
+	local $/ = "\n";
 	my $cmd = '/usr/bin/env bash -c "read -p \"'.shellmeta(shellmeta($prompt)).'\" -r -e && echo -n \"\$REPLY\" 2>/dev/null"';
 	$_ = `$cmd`;
 	return (not $?)? $_: undef;
 }
+sub bashReadLine
+{
+	return bash_readline(@_);
+}
 
-=head3 commandArgs([$prefs, ]@argv)
+=head3 cmdargs([$prefs, ]@argv)
+
+B<commandArgs([$prefs, ]@argv)> I<WILL BE DEPRECATED>
+
+B<cmdArgs([$prefs, ]@argv)> I<WILL BE DEPRECATED>
 
 resolves command line arguments
 
@@ -294,12 +316,8 @@ return value: eg;
 	{ -opt1 => '', -opt2 => 'val2', command => 'cmd', parameters => ['param1', 'param2', 'param3'] }
 	{ -opt1 => '', -opt2 => '', command => 'cmd', parameters => ['param1', 'param2', 'param3'] }
 
-=head3 cmdArgs([$prefs, ]@argv)
-
-synonym with B<commandArgs()>
-
 =cut
-sub commandArgs
+sub cmdargs
 {
 	my $prefs = {};
 	$prefs = shift if @_ >= 1 and ref($_[0]) eq 'HASH';
@@ -370,13 +388,18 @@ sub commandArgs
 
 	return \%result;
 }
-
+sub commandArgs
+{
+	return cmdargs(@_);
+}
 sub cmdArgs
 {
-	return commandArgs(@_);
+	return cmdargs(@_);
 }
 
-=head3 whereisBin($name, $path)
+=head3 whereis($name, $path)
+
+B<whereisBin($name, $path)> I<WILL BE DEPRECATED>
 
 searches valid binary in search path
 
@@ -387,15 +410,21 @@ $path: I<search path, by default "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/
 return value: I<array of binary path founded in search path>
 
 =cut
-sub whereisBin
+sub whereis
 {
 	my ($name, $path) = @_;
 	return () unless $name;
 	$path = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin" unless $path;
 	return grep(-x $_, map("$_/$name", split(":", $path)));
 }
+sub whereisBin
+{
+	return whereis(@_);
+}
 
-=head3 fileCache($tag, $expiry, $subref)
+=head3 file_cache($tag, $expiry, $subref)
+
+B<fileCache($tag, $expiry, $subref)> I<WILL BE DEPRECATED>
 
 gets most recent cached value in file cache by given tag and caller function if there is cached value in expiry period. Otherwise tries to get current value using $subref, puts value in cache and cleanups old cache values.
 
@@ -413,12 +442,12 @@ E<gt>0: I<gets most recent cached value in cache if there is cached value in exp
 
 =back
 
-$subref: I<sub reference to get current value>
+$subref: I<sub reference in CodeRef to get current value>
 
 return value: I<cached or current value, otherwise undef if there isn't cached value and current value doesn't get>
 
 =cut
-sub fileCache
+sub file_cache
 {
 	my ($tag, $expiry, $subref) = @_;
 	my $result;
@@ -427,7 +456,7 @@ sub fileCache
 	my $caller = (caller(1))[3];
 	$caller = (caller(0))[0] unless $caller;
 	$caller = (caller(0))[3].",$caller";
-	my $tagEncoded = "";
+	my $tag_encoded = "";
 	for (0..(bytes::length($tag)-1))
 	{
 		my $c = bytes::substr($tag, $_, 1);
@@ -435,22 +464,22 @@ sub fileCache
 		{
 			$c = uc(sprintf("%%%x", bytes::ord($c)));
 		}
-		$tagEncoded .= $c;
+		$tag_encoded .= $c;
 	}
-	my $tmpBase = "/tmp/";
-	my $tmpPrefix = $caller;
-	$tmpPrefix =~ s/\Q::\E/-/g;
-	$tmpPrefix .= ".$tagEncoded,";
-	for my $tmpPath (sort {$b cmp $a} glob("${tmpBase}$tmpPrefix*"))
+	my $tmp_base = "/tmp/";
+	my $tmp_prefix = $caller;
+	$tmp_prefix =~ s/\Q::\E/-/g;
+	$tmp_prefix .= ".$tag_encoded,";
+	for my $tmp_path (sort {$b cmp $a} glob("${tmp_base}$tmp_prefix*"))
 	{
-		if (my ($epoch, $pid) = $tmpPath =~ /^\Q${tmpBase}$tmpPrefix\E(\d*)\.(\d*)/)
+		if (my ($epoch, $pid) = $tmp_path =~ /^\Q${tmp_base}$tmp_prefix\E(\d*)\.(\d*)/)
 		{
 			if ($expiry < 0 or ($expiry > 0 and $now-$epoch < $expiry))
 			{
 				if (not defined($result))
 				{
 					my $tmp;
-					$tmp = file_get_contents($tmpPath);
+					$tmp = file_get_contents($tmp_path);
 					if ($tmp)
 					{
 						if ($tmp =~ /^SCALAR\n(.*)/)
@@ -465,7 +494,7 @@ sub fileCache
 				next;
 			}
 		}
-		unshift @cleanup, $tmpPath;
+		unshift @cleanup, $tmp_path;
 	}
 	if (not defined($result))
 	{
@@ -480,7 +509,7 @@ sub fileCache
 			{
 				eval { $tmp = to_json($result, {utf8 => 1, pretty => 1}) } if ref($result) eq "ARRAY" or ref($result) eq "HASH";
 			}
-			if ($tmp and file_put_contents("${tmpBase}tmp.$tmpPrefix$now.$$", $tmp) and rename("${tmpBase}tmp.$tmpPrefix$now.$$", "${tmpBase}$tmpPrefix$now.$$"))
+			if ($tmp and file_put_contents("${tmp_base}tmp.$tmp_prefix$now.$$", $tmp) and rename("${tmp_base}tmp.$tmp_prefix$now.$$", "${tmp_base}$tmp_prefix$now.$$"))
 			{
 				pop @cleanup;
 				for (@cleanup)
@@ -492,12 +521,18 @@ sub fileCache
 	}
 	return $result;
 }
+sub fileCache
+{
+	return file_cache(@_);
+}
 
-=head3 getPodText($fileName, $section, $exclude_section)
+=head3 get_pod_text($file_name, $section, $exclude_section)
+
+B<getPodText($file_name, $section, $exclude_section)> I<WILL BE DEPRECATED>
 
 gets a text of pod contents in given file
 
-$fileName: I<file name of searching pod, by default running file>
+$file_name: I<file name of searching pod, by default running file>
 
 $section: I<searching head1 section of pod, by default undef gets all of contents>
 
@@ -506,15 +541,15 @@ $exclude_section: I<excludes section name, by default undef>
 return value: I<text of pod in string or array by line, otherwise undef if an error occurs>
 
 =cut
-sub getPodText
+sub get_pod_text
 {
-	my ($fileName, $section, $exclude_section) = @_;
-	$fileName = "$FindBin::Bin/$FindBin::Script" unless $fileName;
-	return unless -e $fileName;
+	my ($file_name, $section, $exclude_section) = @_;
+	$file_name = "$FindBin::Bin/$FindBin::Script" unless $file_name;
+	return unless -e $file_name;
 	my $parser = Pod::Simple::Text->new();
 	my $text;
 	$parser->output_string(\$text);
-	eval { $parser->parse_file($fileName) };
+	eval { $parser->parse_file($file_name) };
 	return if $@;
 	utf8::decode($text);
 	$section = ltrim($section) if $section;
@@ -547,6 +582,239 @@ sub getPodText
 	return @result if wantarray;
 	return $result;
 }
+sub getPodText
+{
+	return get_pod_text(@_);
+}
+
+=head3 term_readline($prompt, $default, $history)
+
+reads a line from STDIN
+
+$prompt: I<prompt, by default ''>
+
+$default: I<initial value of line, by default ''>
+
+$history: I<lines history in ArrayRef, by default undef>
+
+return value: I<line>
+
+=cut
+sub term_readline
+{
+	my ($prompt, $default, $history) = @_;
+	$prompt = "" unless defined($prompt);
+	$default = "" unless defined($default);
+	my ($in, $out) = (\*STDIN, \*STDOUT);
+	unless (-t *$in)
+	{
+		my $line = <*$in>;
+		chomp $line if defined $line;
+		return $line;
+	}
+	local $\ = undef;
+
+	my $old_sigint = $SIG{INT};
+	local $SIG{INT} = sub {
+		Term::ReadKey::ReadMode('restore', $in);
+		if (defined($old_sigint))
+		{
+			$old_sigint->();
+		} else
+		{
+			print "\n";
+			exit 130;
+		}
+	};
+	my $old_sigterm = $SIG{TERM};
+	local $SIG{TERM} = sub {
+		Term::ReadKey::ReadMode('restore', $in);
+		if (defined($old_sigterm))
+		{
+			$old_sigterm->();
+		} else
+		{
+			print "\n";
+			exit 143;
+		}
+	};
+	Term::ReadKey::ReadMode('cbreak', $in);
+
+	my ($line, $index, $history_index) = ("", 0);
+
+	my $write = sub {
+		my ($text) = @_;
+		my $s;
+		$s = "";
+		for my $c (split(/(.)/, $text))
+		{
+			given ($c)
+			{
+				when (/[\x00-\x1F]/)
+				{
+					$c = "^".chr(0x40+ord($c));
+				}
+				when ($c =~ /[\x7F]/)
+				{
+					$c = "^".chr(0x3F);
+				}
+			}
+			$s .= $c;
+		}
+		$text = $s;
+		substr($line, $index) = $text.substr($line, $index);
+		$index += length($text);
+		print $out "\e[0J";
+		print $out $text;
+		$s = substr($line, $index);
+		print $out $s;
+		print $out "\e[D" x length($s);
+	};
+	my $set = sub {
+		my ($text) = @_;
+		print $out "\e[D" x $index;
+		$index = 0;
+		print $out "\e[0J";
+		$line = "";
+		$write->($text);
+	};
+	my $backspace = sub {
+		my $s;
+		return if $index <= 0;
+		$index--;
+		substr($line, $index, 1) = "";
+		print $out "\e[D\e[0J";
+		$s = substr($line, $index);
+		print $out $s;
+		print $out "\e[D" x length($s);
+	};
+	my $delete = sub {
+		my $s;
+		substr($line, $index, 1) = "";
+		print $out "\e[0J";
+		$s = substr($line, $index);
+		print $out $s;
+		print $out "\e[D" x length($s);
+	};
+	my $home = sub {
+		print $out "\e[D" x $index;
+		$index = 0;
+	};
+	my $end = sub {
+		my $s;
+		print $out "\e[0J";
+		$s = substr($line, $index);
+		print $out $s;
+		$index += length($s);
+	};
+	my $left = sub {
+		return if $index <= 0;
+		print $out "\e[D";
+		$index--;
+	};
+	my $right = sub {
+		return if $index >= length($line);
+		print $out substr($line, $index, 1);
+		$index++;
+	};
+	my $up = sub {
+		return unless ref($history) eq "ARRAY";
+		return if $history_index <= 0;
+		$history->[$history_index] = $line;
+		$history_index--;
+		$set->($history->[$history_index]);
+	};
+	my $down = sub {
+		return unless ref($history) eq "ARRAY";
+		return if $history_index >= $#$history;
+		$history->[$history_index] = $line;
+		$history_index++;
+		$set->($history->[$history_index]);
+	};
+
+	print $prompt;
+	$set->($default);
+	if (ref($history) eq "ARRAY")
+	{
+		push @$history, $line;
+		$history_index = $#$history;
+	}
+
+	my ($char, $esc) = ("", undef);
+	while (read($in, $char, 1))
+	{
+		unless (defined($esc))
+		{
+			given ($char)
+			{
+				when (/\e/)
+				{
+					$esc = "";
+				}
+				when (/\t/)
+				{
+				}
+				when (/\n|\r/)
+				{
+					print $out $char;
+					$history->[$#$history] = $line if ref($history) eq "ARRAY";
+					last;
+				}
+				when (/[\b]|\x7F/)
+				{
+					$backspace->();
+				}
+				default
+				{
+					$write->($char);
+				}
+			}
+			next;
+		}
+		$esc .= $char;
+		if ($esc =~ /^.\d?\D/)
+		{
+			given ($esc)
+			{
+				when (/^\[A/)
+				{
+					$up->();
+				}
+				when (/^\[B/)
+				{
+					$down->();
+				}
+				when (/^\[C/)
+				{
+					$right->();
+				}
+				when (/^\[D/)
+				{
+					$left->();
+				}
+				when (/^\[3~/)
+				{
+					$delete->();
+				}
+				when (/^\[H/)
+				{
+					$home->();
+				}
+				when (/^\[F/)
+				{
+					$end->();
+				}
+				default
+				{
+					$write->("\e$esc");
+				}
+			}
+			$esc = undef;
+		}
+	}
+	Term::ReadKey::ReadMode('restore', $in);
+	return $line;
+}
 
 
 1;
@@ -578,6 +846,10 @@ JSON
 
 Pod::Simple::Text
 
+=item *
+
+Term::ReadKey
+
 =back
 
 =head1 REPOSITORY
